package Sentinel::Tests::Base;

use Sentinel::Base;
@ISA = qw( Sentinel::Base );

use IO::Socket::INET;

%AUTOFIELDS = (
	message => undef,
	host => undef,
	args => undef,

	probe_code => undef,
	probe_message => undef,
	probe_port => 9122,
	lib_host => '0.0.0.0',
	lib_port => 9123
);

sub execute
{
	my $self = shift;
	$self -> logmsg( "Plugin '" . ref( $self ) . "' did not provide an execute method" );
	$self -> logmsg( "  args --> ('" .
		join( "','", @_ ) .
		"')" );
	$self -> message( "Plugin did not provide an execute method" );
	return( TEST_FAIL );
}

sub host_addr
{
	return( shift() -> host() -> addr() );
}

sub probe
{
	# Takes the probe name, returns code/message
	# If no probe name passed, default is the same as the plugin name
	my $self = shift;
	my $probe = shift();
	my( $s, $line );

	if( ! $probe )
	{
		$probe = ref( $self );
		$probe =~ s/^.*://;
	}
	$self -> logmsg( "Probe is: $probe" );

	# Stupid IO::Socket::INET tried to throw a croak
	eval{
		local $SIG{ __DIE__ } = sub { 0 };
		$s = IO::Socket::INET -> new(
			PeerAddr => $self -> host_addr(),
			PeerPort => $self -> probe_port(),
			Proto => 'tcp' );
	};

	if( ! $s )
	{
		$self -> logmsg( "Couldn't create socket: $!" );
		$self -> probe_code( 501 ),
		$self -> probe_message( "Couldn't create socket: $!" );
		return( $self -> probe_code() );
	}

	# OK, do conversation
	# TODO: Fix version info
	$s -> print( "VERSION\r\n" );
	$line = $s -> getline(); # TODO: Validate version string

	# Instruct what and where to download
	$s -> print( "$probe " . $self -> lib_host() . ":" . $self -> lib_port() . "\r\n" );

	# Status of loading probe
	$line = $s -> getline(); 
	chomp( $line );
	$line =~ s/\r$//;
	( $code, $message ) = $line =~ /^(\d+)\s+(\S.+)/;
	if( $code != 200 )
	{
		# Error
		$self -> logmsg( "Error: $code/$message" );
		$self -> probe_code( $code );
		$self -> probe_message( $message );
		return( $self -> probe_code() );
	}

	# Pass arguments
	$s -> print( join( "\t", @_ ) ) if @_;
	$s -> print( "\r\n" );

	# Returns final message code (this is the real probe)
	$line = $s -> getline(); 
	chomp( $line );
	$line =~ s/\r$//;
	( $code, $message ) = $line =~ /^(\d+)\s+(\S.+)/;

	# Final message code
	$self -> logmsg( "Final return of probe: $code/$message" );
	$self -> probe_code( $code );
	$self -> probe_message( $message );
	return( $self -> probe_code() );
}

sub opt
{
	return( shift() -> args() -> { shift() } );
}

1;
