package Sentinel::Tests::loadavg;

use Sentinel::Tests::Base;
@ISA = qw( Sentinel::Tests::Base );

%AUTOFIELDS = (
	retcode => undef 
);

sub execute
{
	my $self = shift;
	my( $one, $five, $fifteen, $running, $total );
	my $foo = $self -> probe();

	if( $foo == 200 )
	{
		# Validate any loadavg data
		( $one, $five, $fifteen, $running, $total ) = split( /,/, $self -> probe_message() );

		$self -> retcode( 0 ); # say OK
		$self -> message( "" ); # Make -w happy

		# Now test each one.  If failure, it will add to message and set retcode,
		# If retcode is one by the end it failed somewhere
		$self -> invalid_value( 'avg1', $one, "Load average too high in last 1 minute" );
		$self -> invalid_value( 'avg5', $five, "Load average too high in last 5 minutes" );
		$self -> invalid_value( 'avg15', $fifteen, "Load average too high in last 15 minutes" );
		$self -> invalid_value( 'running', $running, "Too many running processes" );
		$self -> invalid_value( 'total', $total, "Too many total processes" );

		return( $self -> retcode() );
	}

	$self -> message( $self -> probe_message() );
	return( 1 );
}

sub invalid_value
{
	my $self = shift;
	my $field = shift;
	my $value = shift;
	my $message = shift;

	if( ! defined( $self -> opt( $field ) ) )
	{
		# Was not set, do not check
		return();
	}

	if( $value > $self -> opt( $field ) )
	{
		$self -> message( $self -> message() . 
			$message . 
			": $value max " . 
			$self -> opt( $field ) .
			"\n" );

		# Flag error
		$self -> retcode( 1 );
	}
}

1;
